

import logging
import os
import sqlite3
from hashlib import sha256


class Suppressable(object):
    def __init__(self, db_file=None, logger=None):
        if db_file:
            self.db_file = db_file
        else:
            self.db_file = "/tmp/caosadvanced_suppressed_cache.db"

        if logger:
            self.logger = logger
        else:
            self.logger = logging.getLogger()

        if not os.path.exists(self.db_file):
            self.create_cache()

    def create_cache(self):
        conn = sqlite3.connect(self.db_file)
        c = conn.cursor()
        c.execute('''CREATE TABLE messages (digest text primary key, category text)''')
        conn.commit()
        conn.close()

    def tag_msg(self, txt, identifier, category):
        digest = self.hash(txt, identifier)
        conn = sqlite3.connect(self.db_file)
        c = conn.cursor()
        c.execute('''INSERT INTO messages VALUES (?,?)''', (digest, category))
        conn.commit()
        conn.close()

    def reset(self, category):
        conn = sqlite3.connect(self.db_file)
        c = conn.cursor()
        c.execute('''DELETE FROM messages WHERE category=?''',
                  (category,))
        conn.commit()

    def was_tagged(self, digest):
        conn = sqlite3.connect(self.db_file)
        c = conn.cursor()
        c.execute('''Select  * FROM messages WHERE digest=?''',
                  (digest,))
        res = c.fetchone()
        conn.commit()
        conn.close()

        if res is None:
            return False
        else:
            return True

    def warning(self, txt, identifier, category):
        self.msg(txt, identifier, "warning", category)

    def debug(self, txt, identifier, category):
        self.msg(txt, identifier, "debug", category)

    def info(self, txt, identifier, category):
        self.msg(txt, identifier, "info", category)

    def error(self, txt, identifier, category):
        self.msg(txt, identifier, "error", category)

    def hash(self, txt, identifier):
        return sha256((txt+str(identifier)).encode("utf-8")).hexdigest()

    def msg(self, txt, identifier, kind, category):
        if self.was_tagged(self.hash(txt, identifier)):
            return

        if kind == "debug":
            self.logger.debug(txt)
        elif kind == "info":
            self.logger.info(txt)
        elif kind == "warning":
            self.logger.warning(txt)
        elif kind == "error":
            self.logger.error(txt)

        self.tag_msg(txt, identifier, category)
